]> sigrok.org Git - libsigrok.git/blobdiff - strutil.c
sr/cli/gtk/qt/: s/plugin/driver/.
[libsigrok.git] / strutil.c
index 380d78f59efdbb0bf72009c903484a85d9f87a06..57b4aa88e4532133f9409d0d46a41410ea668605 100644 (file)
--- a/strutil.c
+++ b/strutil.c
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sigrok.h>
-#include <sigrok-internal.h>
+#include "sigrok.h"
+#include "sigrok-internal.h"
 
 /**
  * Convert a numeric samplerate value to its "natural" string representation.
  *
- * E.g. a value of 3000000 would be converted to "3 MHz", 20000 to "20 kHz".
+ * E.g. a value of 3000000 would be converted to "3 MHz", 20000 to "20 kHz",
+ * 31500 would become "31.5 kHz".
  *
  * @param samplerate The samplerate in Hz.
* @return A malloc()ed string representation of the samplerate value,
*         or NULL upon errors. The caller is responsible to free() the memory.
  */
-char *sr_samplerate_string(uint64_t samplerate)
+SR_API char *sr_samplerate_string(uint64_t samplerate)
 {
        char *o;
-       int r;
-
-       o = malloc(30 + 1); /* Enough for a uint64_t as string + " GHz". */
-       if (!o)
-               return NULL;
+       uint64_t s = samplerate;
 
-       if (samplerate >= SR_GHZ(1))
-               r = snprintf(o, 30, "%" PRIu64 " GHz", samplerate / 1000000000);
-       else if (samplerate >= SR_MHZ(1))
-               r = snprintf(o, 30, "%" PRIu64 " MHz", samplerate / 1000000);
-       else if (samplerate >= SR_KHZ(1))
-               r = snprintf(o, 30, "%" PRIu64 " kHz", samplerate / 1000);
-       else
-               r = snprintf(o, 30, "%" PRIu64 " Hz", samplerate);
+       if ((s >= SR_GHZ(1)) && (s % SR_GHZ(1) == 0)) {
+               o = g_strdup_printf("%" PRIu64 " GHz", s / SR_GHZ(1));
+       } else if ((s >= SR_GHZ(1)) && (s % SR_GHZ(1) != 0)) {
+               o = g_strdup_printf("%" PRIu64 ".%" PRIu64 " GHz",
+                                   s / SR_GHZ(1), s % SR_GHZ(1));
+       } else if ((s >= SR_MHZ(1)) && (s % SR_MHZ(1) == 0)) {
+               o = g_strdup_printf("%" PRIu64 " MHz", s / SR_MHZ(1));
+       } else if ((s >= SR_MHZ(1)) && (s % SR_MHZ(1) != 0)) {
+               o = g_strdup_printf("%" PRIu64 ".%" PRIu64 " MHz",
+                                   s / SR_MHZ(1), s % SR_MHZ(1));
+       } else if ((s >= SR_KHZ(1)) && (s % SR_KHZ(1) == 0)) {
+               o = g_strdup_printf("%" PRIu64 " kHz", s / SR_KHZ(1));
+       } else if ((s >= SR_KHZ(1)) && (s % SR_KHZ(1) != 0)) {
+               o = g_strdup_printf("%" PRIu64 ".%" PRIu64 " kHz",
+                                   s / SR_KHZ(1), s % SR_KHZ(1));
+       } else {
+               o = g_strdup_printf("%" PRIu64 " Hz", s);
+       }
 
-       if (r < 0) {
-               /* Something went wrong... */
-               free(o);
+       if (!o) {
+               sr_err("strutil: %s: Error creating samplerate string.",
+                      __func__);
                return NULL;
        }
 
@@ -67,17 +76,21 @@ char *sr_samplerate_string(uint64_t samplerate)
  * E.g. a value of 3000000 would be converted to "3 us", 20000 to "50 ms".
  *
  * @param frequency The frequency in Hz.
* @return A malloc()ed string representation of the frequency value,
*         or NULL upon errors. The caller is responsible to free() the memory.
  */
-char *sr_period_string(uint64_t frequency)
+SR_API char *sr_period_string(uint64_t frequency)
 {
        char *o;
        int r;
 
-       o = malloc(30 + 1); /* Enough for a uint64_t as string + " ms". */
-       if (!o)
+       /* Allocate enough for a uint64_t as string + " ms". */
+       if (!(o = g_try_malloc0(30 + 1))) {
+               sr_err("strutil: %s: o malloc failed", __func__);
                return NULL;
+       }
 
        if (frequency >= SR_GHZ(1))
                r = snprintf(o, 30, "%" PRIu64 " ns", frequency / 1000000000);
@@ -90,7 +103,7 @@ char *sr_period_string(uint64_t frequency)
 
        if (r < 0) {
                /* Something went wrong... */
-               free(o);
+               g_free(o);
                return NULL;
        }
 
@@ -100,12 +113,13 @@ char *sr_period_string(uint64_t frequency)
 /**
  * TODO
  *
- * @param device TODO
+ * @param dev TODO
  * @param triggerstring TODO
+ *
  * @return TODO
  */
-char **sr_parse_triggerstring(struct sr_device *device,
-                             const char *triggerstring)
+SR_API char **sr_parse_triggerstring(struct sr_dev *dev,
+                                    const char *triggerstring)
 {
        GSList *l;
        struct sr_probe *probe;
@@ -113,7 +127,7 @@ char **sr_parse_triggerstring(struct sr_device *device,
        char **tokens, **triggerlist, *trigger, *tc, *trigger_types;
        gboolean error;
 
-       max_probes = g_slist_length(device->probes);
+       max_probes = g_slist_length(dev->probes);
        error = FALSE;
 
        if (!(triggerlist = g_try_malloc0(max_probes * sizeof(char *)))) {
@@ -122,7 +136,7 @@ char **sr_parse_triggerstring(struct sr_device *device,
        }
 
        tokens = g_strsplit(triggerstring, ",", max_probes);
-       trigger_types = device->plugin->get_device_info(0, SR_DI_TRIGGER_TYPES);
+       trigger_types = dev->driver->dev_info_get(0, SR_DI_TRIGGER_TYPES);
        if (trigger_types == NULL)
                return NULL;
 
@@ -130,7 +144,7 @@ char **sr_parse_triggerstring(struct sr_device *device,
                if (tokens[i][0] < '0' || tokens[i][0] > '9') {
                        /* Named probe */
                        probenum = 0;
-                       for (l = device->probes; l; l = l->next) {
+                       for (l = dev->probes; l; l = l->next) {
                                probe = (struct sr_probe *)l->data;
                                if (probe->enabled
                                    && !strncmp(probe->name, tokens[i],
@@ -166,8 +180,7 @@ char **sr_parse_triggerstring(struct sr_device *device,
 
        if (error) {
                for (i = 0; i < max_probes; i++)
-                       if (triggerlist[i])
-                               g_free(triggerlist[i]);
+                       g_free(triggerlist[i]);
                g_free(triggerlist);
                triggerlist = NULL;
        }
@@ -186,19 +199,19 @@ char **sr_parse_triggerstring(struct sr_device *device,
  * Spaces (but not other whitespace) between value and suffix are allowed.
  *
  * @param sizestring A string containing a (decimal) size value.
- * @return The string's size value as uint64_t.
+ * @param size Pointer to uint64_t which will contain the string's size value.
  *
- * TODO: Error handling.
+ * @return SR_OK upon success, SR_ERR upon errors.
  */
-uint64_t sr_parse_sizestring(const char *sizestring)
+SR_API int sr_parse_sizestring(const char *sizestring, uint64_t *size)
 {
-       int multiplier;
-       uint64_t val;
+       int multiplier, done;
        char *s;
 
-       val = strtoull(sizestring, &s, 10);
+       *size = strtoull(sizestring, &s, 10);
        multiplier = 0;
-       while (s && *s && multiplier == 0) {
+       done = FALSE;
+       while (s && *s && multiplier == 0 && !done) {
                switch (*s) {
                case ' ':
                        break;
@@ -215,15 +228,18 @@ uint64_t sr_parse_sizestring(const char *sizestring)
                        multiplier = SR_GHZ(1);
                        break;
                default:
-                       val = 0;
-                       multiplier = -1;
+                       done = TRUE;
+                       s--;
                }
                s++;
        }
        if (multiplier > 0)
-               val *= multiplier;
+               *size *= multiplier;
+
+       if (*s && strcasecmp(s, "Hz"))
+               return SR_ERR;
 
-       return val;
+       return SR_OK;
 }
 
 /**
@@ -245,7 +261,7 @@ uint64_t sr_parse_sizestring(const char *sizestring)
  * TODO: picoseconds?
  * TODO: Allow both lower-case and upper-case.
  */
-uint64_t sr_parse_timestring(const char *timestring)
+SR_API uint64_t sr_parse_timestring(const char *timestring)
 {
        uint64_t time_msec;
        char *s;
@@ -268,17 +284,16 @@ uint64_t sr_parse_timestring(const char *timestring)
        return time_msec;
 }
 
-gboolean sr_parse_boolstring(const char *boolstr)
+SR_API gboolean sr_parse_boolstring(const char *boolstr)
 {
        if (!boolstr)
                return FALSE;
 
-       if (!g_strcasecmp(boolstr, "true") || 
+       if (!g_strcasecmp(boolstr, "true") ||
            !g_strcasecmp(boolstr, "yes") ||
            !g_strcasecmp(boolstr, "on") ||
-           !g_strcasecmp(boolstr, "1")) 
+           !g_strcasecmp(boolstr, "1"))
                return TRUE;
 
        return FALSE;
 }
-
@@
  *
  * @param samplerate The samplerate in Hz.
- * @return A malloc()ed string representation of the samplerate value,
- *         or NULL upon errors. The caller is responsible to free() the memory.
+ *
+ * @return A g_try_malloc()ed string representation of the samplerate value,
+ *         or NULL upon errors. The caller is responsible to g_free() the
+ *         memory.
  */
@@ -67,17 +76,21 @@ char *sr_samplerate_string(uint64_t samplerate)
  *
  * @param frequency The frequency in Hz.
- * @return A malloc()ed string representation of the frequency value,
- *         or NULL upon errors. The caller is responsible to free() the memory.
+ *
+ * @return A g_try_malloc()ed string representation of the frequency value,
+ *         or NULL upon errors. The caller is responsible to g_free() the
+ *         memory.
  */